function [a,beta,error,rightEye] = curveFitting(X, Y, W, xBounds, yBounds, step, a0, beta0, method)

opts1= optimset('display','off');

switch method
    case 1 
        %% Use lsqnonlin (least squares non-linear optimization)
        % to find parameters a and beta of double-parabola
        C0=[a0;beta0];
        coeff = lsqnonlin(@doubleparabolicFunc, C0, [], [], opts1, X, Y, W);
        a = coeff(1);
        beta = coeff(2);    
        
        % Determine if vessels belong to right or left eye
        rightHalf = X>=Y/(tan(-beta+pi/2));
        leftHalf = X<Y/(tan(-beta+pi/2));
        rightEye = sum(W(rightHalf))<sum(W(leftHalf));  
        
        % Reformulate angle for right eye
        if (rightEye),
            if (beta>0),
                beta = beta-pi;
            else
                beta = beta+pi;
            end
        end
        
        % Compute point set of estimated curve and determine approximation error
        [x_new,y_new] = doubleParabolaPoints(a, beta, xBounds, yBounds, step);
        errorVector = zeros(1,length(X));
        for i=1:length(X),
            errorVector(i) = W(i)*point2curveDist(X(i), Y(i), x_new, y_new);
        end
        error = norm(errorVector);
           
    case 2 
        %% Use lsqnonlin (least squares non-linear optimization) 
        % to find parameters a and ? of parabola:  
        C0=[a0;beta0];
        
        coeff = lsqnonlin(@parabolicFunc, C0, [], [], opts1, X, Y, W);
        a = coeff(1);
        beta = coeff(2);   
        
        % Ensure that a is positive by flipping angle if necessary
        if (a<0),
           a = -a;
           if (beta<0),
               beta = beta+pi;
           else
               beta = beta-pi;
           end
        end
        
        % Determine if vessels belong to right or left eye
        rightEye = abs(beta)>pi/2;
           
        % Compute point set of estimated curve and determine approximation error
        [x_new,y_new] = parabolaPoints(a, beta, xBounds, yBounds, step);
        errorVector = zeros(1,length(X));
        for i=1:length(X),
            errorVector(i) = W(i)*point2curveDist(X(i), Y(i), x_new, y_new);
        end
        error = norm(errorVector);

    case 3 
        %% Use SVD to compute coefficients of general parabolic equation
        % c1*x^2+c2*xy+c3*y^2+c4*x+c5*y=0 to determine parameters a and beta
        measurements = [X'.*X', X'.*Y', Y'.*Y', X', Y'];
        disp(['svd: number of data points = ',num2str(size(measurements,1))]);
        [~,~,V] = svd(measurements);
        coeff = V(:,end);
        
        % Determine parameter beta from values of sin(b) and -cos(b)
        if (coeff(5)<=0 && -coeff(4)<0), % -pi <= beta < -pi/2
           beta = -acos(-coeff(4));
        elseif (coeff(5)<0 && -coeff(4)>=0), % -pi/2 <= beta < 0
           beta = asin(coeff(5));
        elseif (coeff(5)>=0 && -coeff(4)>0), % 0 <= beta < pi/2
           beta = asin(coeff(5));
        elseif (coeff(5)>0 && -coeff(4)<=0), % pi/2 <= beta < pi
           beta = acos(-coeff(4));
        end
        if (abs(-cos(beta)-coeff(4))>0.1 || abs(sin(beta)-coeff(5))>0.1),
           disp('Results of svd are not good. Angle can not be computed from coefficients.');
        end
        
        % Determine parameter a from coefficients c1,c2,c3 with known beta
        as = [coeff(1)/(sin(beta))^2, coeff(2)/(2*sin(beta)*cos(beta)), coeff(3)/(cos(beta))^2]
        a = median(as);
        
        % Ensure that a is positive by flipping angle if necessary
        if (a<0),
           a = -a;
           if (beta<0),
               beta = beta+pi;
           else
               beta = beta-pi;
           end
        end
        
        % Determine if vessels belong to right or left eye
        rightEye = abs(beta)>pi/2;
        
        % Compute point set of estimated curve and determine approximation error
        [x_new,y_new] = parabolaPoints(a, beta, xBounds, yBounds, step);
        errorVector = zeros(1,length(X));
        for i=1:length(X),
            errorVector(i) = W(i)*point2curveDist(X(i), Y(i), x_new, y_new);
        end
        error = norm(errorVector);
        
    otherwise
        a = 1;
        beta = 0;
end

% %% Plot results
% if rightEye,
%     eye = 'right';
% else
%     eye = 'left';
% end
% figure, plot(X,Y,'.r',x_new,y_new,'*b'), 
% title(sprintf('beta = %f, %s eye', beta*180/pi,eye)), 
% daspect([1,1,1])

end


function p2cDist = point2curveDist(pX,pY,curveX,curveY)

samples = length(curveX);
errorVectors = [pX*ones(1,samples)-curveX; pY*ones(1,samples)-curveY];
distances = sqrt(errorVectors(1,:).^2 + errorVectors(2,:).^2);
[p2cDist, idx] = min(distances);

end