close all;
clear all;

addpath('HelperFunctions')

%% Parameters
visualization = false;
nrOfParabolaPoints = 100;
nrOfMaculaEstimates = 16;
iteration = 1;

for scale = [1],
    for useGoldStandard = [0,1],  
        
        clearvars -except iteration nrOfParabolaPoints scale useGoldStandard nrOfMaculaEstimates visualization;
        close all;
        
        %% Read input data
        tic;
        imageDirectory = [cd, '/Data/images'];
        if (useGoldStandard),
            vesselDirectory = [cd, '/Data/gs_vessels'];    
        else
            vesselDirectory = [cd, '/Data/vessels']; 
        end
        maskDirectory = [cd, '/Data/mask'];
        imageFilenames = dir( [imageDirectory, '/*.jpg'] );
        if (useGoldStandard),
            vesselFilenames = dir( [vesselDirectory, '/*.tif'] );
        else
            vesselFilenames = dir( [vesselDirectory, '/*.png'] );    
        end
        maskFilenames = dir( [maskDirectory, '/*.tif'] );
        disp('Read input images and vessel segmentation...');

        n = length(imageFilenames)
        files = 1:n

        for k = files

            I_filename = imageFilenames(k).name;
            I_vessels_filename = vesselFilenames(k).name;
            I_mask_filename = maskFilenames(k).name;

            disp( sprintf('...Load %s, %s, %s, %s', I_filename, I_vessels_filename, I_mask_filename) );

            % Get the vessel segmentation
            I_vessels = imread( [vesselDirectory, '/', I_vessels_filename] );
            I_vessels = imresize(I_vessels, scale);
            I_vessels(I_vessels > 0) = 1;
            I_vessels = uint8(I_vessels(:,:,1));

            % Get the image mask
            I_mask = imread( [maskDirectory, '/', I_mask_filename] );
            I_mask = imresize(I_mask, scale);    
            I_mask(I_mask > 0) = 1;
            I_mask = uint8(I_mask);

            % Get current image
            I = imread( [imageDirectory, '/', I_filename] );
            I = imresize(I, scale);
            I(I_mask==0) = 0;	

            % For test scale image
            images(:,:,:,k) = I;
            vessels(:,:,k) = I_vessels;
            masks(:,:,:,k) = I_mask;

        end
        disp('');

        %% Read Gold Standards
        disp('Read gold standard...');
        gs_xlsdata = xlsread('/Data/goldstandards.xls');
        a_xlsdata = xlsread('/Data/results_ODcentersdiameters.xls');
        
        vs_centers = gs_xlsdata(:, 3:4);
        vs_centers = [vs_centers(:,2) vs_centers(:,1)];
        vs_centers = round(scale * vs_centers);
        gs_macula = gs_xlsdata(:, 6:7);
        gs_macula = [gs_macula(:,2) gs_macula(:,1)];
        gs_macula = round(scale * gs_macula);
        gs_rightEye = gs_xlsdata(:, 8); 
        gs_rapheAngle = gs_xlsdata(:, 9); 
        
        if (useGoldStandard),
            gs_centers = gs_xlsdata(:, 1:2);
            gs_centers = [gs_centers(:,2) gs_centers(:,1)];
            gs_centers = round(scale * gs_centers);
            on_diameters = gs_xlsdata(:, 5);
            on_diameters = round(scale * on_diameters);
        else
            gs_centers = a_xlsdata(:, 1:2);
            gs_centers = [gs_centers(:,2) gs_centers(:,1)];
            gs_centers = round(scale * gs_centers);
            on_diameters = a_xlsdata(:, 3);
            on_diameters = round(scale * on_diameters);
        end
        disp('');
        toc;

        for vesselPointSetAlg = 1:7,
            for curvFitAlg = 1:2,  
                
                %% Run Localization
                identifier = ['s',num2str(scale),'_gs',num2str(useGoldStandard),'_v',num2str(vesselPointSetAlg),'_f',num2str(curvFitAlg)];
                disp(['Run macula localization with: ', identifier]);
                resultDir = [cd, '/results/'];
                
                deviationFromGS = zeros(length(files),nrOfMaculaEstimates);
                angleDeviation = zeros(1,length(files));
                rapheAngle = zeros(1,length(files));
                correctEyeEstimate = zeros(1,length(files));
                imgCounter = 1;
                
                for k = files
                    tic;
                    I = images(:,:,:,k);
                    I_vessels = vessels(:,:,k);
                    [~,name,~] = fileparts(imageFilenames(k).name);                    
                    disp( sprintf('Process image %i: %s', k, name) );

                    %% Select points from binary vessel image
                    [vesselX,vesselY,vesselWeights] = vesselPointSetSelection(I_vessels, vesselPointSetAlg, vs_centers(k,:), [resultDir, num2str(k)]);

                    %% Fit parabolic model to data points

                    % Set coordinate origin to vessel origin
                    vesselX = vesselX' - vs_centers(k,2);
                    vesselY = -vesselY' + vs_centers(k,1); 
                    
                    vesselWeights = double(vesselWeights');

                    % Fit Parabola to selected vessel points
                    xBounds = [min(vesselX),max(vesselX)];
                    yBounds = [min(vesselY),max(vesselY)];	
                    dataStep = (max([max(vesselX),max(vesselY)]) - max([min(vesselX),min(vesselY)])) / nrOfParabolaPoints;	
                    [a,beta,error,rightEye] = curveFitting(vesselX, vesselY, vesselWeights, xBounds, yBounds, dataStep,  0.005, 0, curvFitAlg);
                    
                    % Compute angle deviation from goldstandard and check
                    % whether eye was identified correctly (left/right)
                    rapheAngle(k) = beta;
                    if (isnan(gs_rapheAngle(k))),
                        angleDeviation(k) = NaN;
                    elseif (abs(beta-gs_rapheAngle(k))>pi),
                        angleDeviation(k) = abs(abs(beta-gs_rapheAngle(k))-2*pi);
                    else
                        angleDeviation(k) = abs(beta-gs_rapheAngle(k));
                    end
                    correctEyeEstimate(k) = (rightEye==gs_rightEye(k));

                    % Correct estimation if angle between raphe and x-axis is greater than 30
                    if rightEye && (abs(beta)<5/6*pi),
                        betaCorrected = pi;
                    elseif ~rightEye && (abs(beta)>pi/6)
                        betaCorrected = 0;
                    else
                        betaCorrected = beta;
                    end

                    %% Estimate Macula Position

                    % Retrieve first estimate of macula position from model  
                    shiftVector = 2.5 * on_diameters(k) * [sin(betaCorrected),cos(betaCorrected)]; % ON on left side
                    modelEstimate = round(gs_centers(k,:) + shiftVector);

                    % Select ROI for further estimates
                    roiRows = max(modelEstimate(1)-on_diameters(k),1) : max(min(modelEstimate(1)+on_diameters(k) , size(I_vessels,1)),1);
                    roiCols = max(modelEstimate(2)-on_diameters(k),1) : max(min(modelEstimate(2)+on_diameters(k) , size(I_vessels,2)),1);
                    
                    if (length(roiRows)<=1 || length(roiCols)<=1),
                        disp('error: no roi can be selected, estimate from curve fitting is probably outside the image');
                        distmapEstimat = [NaN,NaN];
                        intensityEstimates = NaN(nrOfMaculaEstimates-2,2);
                    else
                        % Retrieve second estimate of macula position from maximum in distance map
                        nonVesselDistMap = bwdist(I_vessels);
                        nonVesselDistMap = nonVesselDistMap(roiRows, roiCols);
                    	[row,col] = find(nonVesselDistMap==max(nonVesselDistMap(:)));
                        distmapEstimate = [row(1),col(1)] + [roiRows(1),roiCols(2)];    

                        % Retrieve third estimate of macula position from image intensity
                        [intensityEstimates, maculaRadii] = intensityAnalysis(I, I_vessels, distmapEstimate, on_diameters(k), name);
                    end
                    
                    %% Analyse and show results

                    % Analyse calculated macula positions with respect to gold standard
                    maculaCenters = [modelEstimate;distmapEstimate;intensityEstimates];
                    if (isnan(gs_macula(k))),
                        deviationFromGS(k,:) = repmat(NaN,1,size(maculaCenters,1));
                    else
                        deviationVector = (maculaCenters-repmat(gs_macula(k,:),size(maculaCenters,1),1))';
                        deviationFromGS(k,:) = sqrt(deviationVector(1,:).^2+deviationVector(2,:).^2);  
                    end    

                    if (visualization),
                        % Mark gold standard ON center and macula center estimates in color image      
                        figure,imshow(I),title(imageFilenames(k).name);   
                        hold on, plot(gs_centers(k,2),gs_centers(k,1),'+k',...
                                      maculaCenters(1,2),maculaCenters(1,1),'+b',...
                                      maculaCenters(2,2),maculaCenters(2,1),'+m',...
                                      maculaCenters(3:end,2),maculaCenters(3:end,1),'+g');
                        if (~isnan(gs_macula(k))),
                           hold on, plot(gs_macula(k,2),gs_macula(k,1),'+k');
                        end

                        % Plot results in vessel coordinate system
                        onCenterX = gs_centers(k,2)-vs_centers(k,2);
                        onCenterY = -(gs_centers(k,1)-vs_centers(k,1));
                        gsMaculaX = gs_macula(k,2)-vs_centers(k,2);
                        gsMaculaY = -(gs_macula(k,1)-vs_centers(k,1));
                        maculaEstimsX = maculaCenters(:,2)-repmat(vs_centers(k,2),size(maculaCenters,1),1);
                        maculaEstimsY = -(maculaCenters(:,1)-repmat(vs_centers(k,1),size(maculaCenters,1),1));
                        rapheX = min(vesselX):max(vesselX);
                        rapheY = tan(-beta)*rapheX + (onCenterY-tan(-beta)*onCenterX);
                        if curvFitAlg==1,
                            [x_new,y_new] = doubleParabolaPoints(a, beta, xBounds, yBounds, dataStep);
                        else
                            [x_new,y_new] = parabolaPoints(a, beta, xBounds, yBounds, dataStep);
                        end
                        figure, plot(vesselX,vesselY,'.r', ...
                                     x_new, y_new, '*b', ...
                                     rapheX,rapheY, '-b', ...                 
                                     0,0,'+k', ...
                                     xBounds,[0,0],'--k',...
                                     [0,0],yBounds,'--k',...
                                     onCenterX, onCenterY, 'vk', ...
                                     gsMaculaX, gsMaculaY, '*k', ...
                                     maculaEstimsX(1), maculaEstimsY(1), 'ob', ...
                                     maculaEstimsX(2), maculaEstimsY(2), 'om', ...
                                     maculaEstimsX(3:end), maculaEstimsY(3:end), 'og');
                        if (rightEye),
                            title(sprintf('beta = %f, right eye', beta*180/pi));
                        else
                            title(sprintf('beta = %f, left eye', beta*180/pi));
                        end
                        axis equal;
                    end
                    
                    % Coordinates of estimated macula position in scaled image
                    xlswrite([resultDir,'maculaCenters_',identifier,'.xls'], {['X_',name];['Y_',name]}, ...
                             ['A',num2str(2*imgCounter),':A',num2str(2*imgCounter+1)]);
                    xlswrite([resultDir,'maculaCenters_',identifier,'.xls'], maculaCenters', ...
                             ['B',num2str(2*imgCounter),':Q',num2str(2*imgCounter+1)]);

                    imgCounter = imgCounter + 1;
                    toc;
                end
                
                xlsrows=[2*iteration,2*iteration+1];
                
                % Analysis of 16 Macula Localization Algorithms by measuring
                % deviation from goldstandard macula posistion in pixels
                meanError = nanmean(deviationFromGS,1)
                stdError = nanstd(deviationFromGS,1,1)
                xlswrite([resultDir,'maculaAlgEval.xls'],{['mean_',identifier];['std_',identifier]},1,['A',num2str(xlsrows(1)),':A',num2str(xlsrows(2))]);
                xlswrite([resultDir,'maculaAlgEval.xls'],meanError,1,['B',num2str(xlsrows(1)),':Q',num2str(xlsrows(1))]);
                xlswrite([resultDir,'maculaAlgEval.xls'],stdError,1,['B',num2str(xlsrows(2)),':Q',num2str(xlsrows(2))]);
                
                % Deviation from gold standard angle (angle between x-axis 
                % and line through macula gold standard and optiv nerve center)
                meanAngleError = nanmean(angleDeviation*180/pi)
                stdAngleError = nanstd(angleDeviation*180/pi)  
                xlswrite([resultDir,'maculaAlgEval.xls'], {identifier}, 2, ['A',num2str(iteration)]);
                xlswrite([resultDir,'maculaAlgEval.xls'], meanAngleError, 2, ['B',num2str(iteration)]);
                xlswrite([resultDir,'maculaAlgEval.xls'], stdAngleError, 2, ['C',num2str(iteration)]);             
                
                % Right/Left Eye Matching Ability: 1 if all images were 
                % correctly matched to right / left eye
                meanEyeError = nanmean(correctEyeEstimate)
                xlswrite([resultDir,'maculaAlgEval.xls'], {identifier}, 3, ['A',num2str(iteration)]);
                xlswrite([resultDir,'maculaAlgEval.xls'], meanEyeError, 3, ['B',num2str(iteration)]);
                
                iteration = iteration+1;

            end
        end
    end
end