function [ dataX , dataY , weights ] = vesselPointSetSelection( vesselImage , method, center, filename, varargin )
%VESSELPOINTSETSELECTION select points from vasculature which represent
%main vessels 
%   Different method (1-3) can be used to select vessel point:
%       method 1: Compute distance transform on vessels and delete points below a threshold
%       method 2: Compute distance transform on vessels and multiply with skeleton
%       method 3: Compute density map of vessels and delete points below a threshold
%       method 4: Compute density map of vessels and multiply with original vessels
%       method 5: Compute density map of vessels and do point sampling via halftoning
%       method 6: Compute density map of vessels and multiply with halftoning of vessels
%       method 7: Extract graph from skeleton, remove short leaves, remove edges which are proportionally thicker 
%                 than the distance to the vessel origin and select main vasculature
%       (method 8: Get every kth pixel in skeleton, compute voronoi
%       diagram and assign mean of voronoi cell to point)


    % Parameters
    radius = round(max(size(vesselImage))/70);
    relDataVolume = 0.2;

    % Preprocessing: Fill holes
    CC = bwconncomp(1-vesselImage,4);
    numPixels = cellfun(@numel,CC.PixelIdxList);
    ind = find(numPixels<10);
    pixels = CC.PixelIdxList(ind);
    vesselImage(cell2mat(pixels(:))) = 1;
    
    % Preprocessing: Remove small unconnected parts
    CC = bwconncomp(vesselImage,4);
    numPixels = cellfun(@numel,CC.PixelIdxList);
    ind = find(numPixels<0.1*max(numPixels));
    pixels = CC.PixelIdxList(ind);
    vesselImage(cell2mat(pixels(:))) = 0;
    
    % Preprocessing: Determine centerline 
    % skeletonize -> enhance skeleton (remove border, fill holes)
    skeleton = bwmorph(vesselImage,'skel',Inf);
    skeleton([1,end],:) = 0;
    skeleton(:,[1,end]) = 0;
    skeletonFilled = imerode(skeleton,[0,1,0;1,0,1;0,1,0]) + skeleton;
    skeletonFilled = bwmorph(skeletonFilled,'majority') + skeletonFilled;
    skeleton = bwmorph(skeletonFilled,'skel',Inf);
    
    vesselImage = vesselImage>0;
    [m,n] = size(vesselImage);
    
    switch method
        case 1 
            %% Compute distance map on vessels and delete points below a threshold
            
            % Distance transform on binary vessel image
            distTrafo = bwdist(1-vesselImage,'cityblock');          
            % Compute histogram count
            h = histc(distTrafo(:),0:max(distTrafo(:)));            
            % Compute cumulative histogram count
            histCum = fliplr(cumsum(fliplr(h')));        
            % Get number of nonzero pixels
            nonZeroPixelCount = histCum(2);            
            % Define Threshold 
            threshold = find(histCum<=relDataVolume*nonZeroPixelCount,1) -1;           
            % Do thresholding on data and retrieve nonzero Points
            vImg = (distTrafo>=threshold).*distTrafo;
            [dataY, dataX, weights] = find(vImg); 
            
            result = scaledata(vImg, 0, 255);
            result = uint8(round(result));
            
        case 2
            %% Compute distance transform on vessels and multiply with skeleton
 
            % Distance transform on binary vessel image
            distTrafo = bwdist(1-vesselImage,'cityblock');                    
            % Select pixels of distance map belonging to skeleton
            vImg = skeleton.*distTrafo;
            [dataY, dataX, weights] = find(vImg); 
            
            result = scaledata(vImg, 0, 255);
            result = uint8(round(result));
            
        case 3 
            %% Compute density map of vessels and delete points below a threshold
            
            % Distance transform on binary vessel image
            densityImg = integaverage(vesselImage, radius);
            % Scale data to range [0,20] and quantize
            densityImg = scaledata(densityImg,0,20);
            densityImg = round(densityImg);
            % Compute histogram count
            h = histc(double(densityImg(:)),0:max(densityImg(:)));    
            % Compute cumulative histogram count
            histCum = fliplr(cumsum(fliplr(h')));
            % Get number of nonzero pixels
            nonZeroPixelCount = histCum(2); 
            % Define Threshold 
            threshold = find(histCum<=relDataVolume*nonZeroPixelCount,1) -1;
            % Do thresholding on data and retrieve nonzero Points
            vImg = (densityImg>=threshold).*densityImg;
            [dataY, dataX, weights] = find(vImg);
            
            result = scaledata(vImg, 0, 255);
            result = uint8(round(result));
            
         case 4 
            %% Compute density map of vessels and multiply with original vessels
            
            % Distance transform on binary vessel image
            densityImg = integaverage(vesselImage, radius);
            % Select pixels of distance map belonging to vessels
            vImg = double(vesselImage).*densityImg;
            [dataY, dataX, weights] = find(vImg);
            
            result = scaledata(vImg, 0, 255);
            result = uint8(round(result));   
            
        case 5
            %% Compute density map of vessels and do point sampling via halftoning
            
            radius = round(max(size(vesselImage))/40);
            densityImg = integaverage(vesselImage, radius);          
            densityImg = scaledata(double(densityImg),0,1);
            stretchedDensity = scaledata(densityImg.^2,0,80);
            halftone = jarvisHalftone(stretchedDensity);
            vImg = densityImg.*halftone;
            [dataY, dataX, weights] = find(vImg);
            
            result = scaledata(vImg, 0, 255);
            result = uint8(round(result));
            
        case 6
            %% Compute density map of vessels and multiply with halftoning of vessels
            
            radius = round(max(size(vesselImage))/40);
            densityImg = integaverage(vesselImage, radius);
            g = fspecial('gaussian',[radius,radius],radius/5);
            blurredVessels = imfilter(double(vesselImage),g,'same','replicate');
            densityImg = double(blurredVessels).*densityImg; 
            densityImg = scaledata(double(densityImg),0,1);
            stretchedDensity = scaledata(densityImg.^2,0,128);
            halftone = jarvisHalftone(stretchedDensity);
            vImg = densityImg.*halftone;
            [dataY, dataX, weights] = find(vImg);
            
            result = scaledata(vImg, 0, 255);
            result = uint8(round(result));

        case 7 
            %% Extract graph from skeleton, remove short leaves, remove 
            % edges which are proportionally thicker than the distance 
            % to the vessel origin and select main vasculature

            branchpoints = bwmorph(skeleton,'branchpoints');
            endpoints = bwmorph(skeleton,'endpoints');
            poi = (branchpoints+endpoints)>0;

            poiDilated = zeros(m,n);
            strels(:,:,1) = [0,0,0;1,0,1;0,0,0];
            strels(:,:,2) = [0,0,1;1,0,0;0,0,0];
            strels(:,:,3) = [0,0,0;1,0,0;0,0,1];
            strels(:,:,4) = [1,0,0;0,0,1;0,0,0];
            strels(:,:,5) = [0,0,0;0,0,1;1,0,0];
            for i=1:size(strels,3),
                poiDilated = poiDilated + bwhitmiss(poi,strels(:,:,i),1-strels(:,:,i));
                poiDilated = poiDilated + bwhitmiss(poi,strels(:,:,i)',1-strels(:,:,i)');
            end
            poiDilated = imdilate(poiDilated+poi,[0,1,0;1,1,1;0,1,0]) >0;
            poiDilated([1,end],:) = 0;
            poiDilated(:,[1,end]) = 0;
            
            edgeImg = skeleton;
            edgeImg(poiDilated) = 0;
            [L, num] = bwlabel(edgeImg,8);

            neighbors = [ 1,0 ; 0,1 ; -1,0 ; 0,-1 ];
            [r,c] = find(poiDilated);
            labeledEdges = L;
            pointsLabeled = zeros(m,n);

            % Fill gaps between branch/end points and edges
            for i=1:length(r),
                pointsLabeled(r(i),c(i)) = i;
                for k=1:4,
                    gap = neighbors(k,:)+[r(i),c(i)];
                    roi = [max(gap(1)-1,1), max(gap(2)-1,1), min(gap(1)+1,m), min(gap(2)+1,n)];
                    edgeLabels = nonzeros(L(roi(1):roi(3),roi(2):roi(4))); % bottom pixel
                    if ~isempty(edgeLabels), 
                        labeledEdges(gap(1),gap(2)) = skeleton(gap(1),gap(2)) * edgeLabels(1); 
                    end
                end
            end

            edges = cell(num,1);
            points = cell(length(r),1);

            % Retrieve connection between edges and branch points
            for i=1:length(r),
                edgesOfPoi = nonzeros(labeledEdges(r(i)-1:r(i)+1,c(i)-1:c(i)+1));
                %disp([i,r(i),c(i),edgesOfPoi']);
                for k=1:length(edgesOfPoi),
                   edges{edgesOfPoi(k)} = [edges{edgesOfPoi(k)},i]; 
                end
                points{i} = edgesOfPoi;
            end

            % Collapse point clusters to single points
            CC = bwconncomp(poiDilated,8);
            numPixels = cellfun(@numel,CC.PixelIdxList);
            ind = find(numPixels>1);
            pixels = CC.PixelIdxList(ind);
            deletedPoints = [];
            for i=1:length(ind),
                pointCluster = pointsLabeled(pixels{i});
                edgeInd = cell2mat(points(pointCluster));
                for ii = 1:length(edgeInd),
                    edges{edgeInd(ii)}(ismember(edges{edgeInd(ii)},pointCluster)) = pointCluster(1);
                end 
                deletedPoints = [deletedPoints,pointCluster(2:end)'];
            end

            for i=1:num,
                edges{i} = unique(edges{i});
            end

            % Build adjacency matric with 3 values per edge (edge number,
            % edge length in pixels, mean edge radius)
            distTrafo = bwdist(1-vesselImage,'cityblock');
            adjacencyMatrix = zeros(length(points),length(points),3);
            for e=1:length(edges),
                if (length(edges{e})==2),
                    vesselRadius = mean(mean((labeledEdges==e).*distTrafo));
                    adjacencyMatrix(edges{e}(1),edges{e}(2),:) = [e,nnz(labeledEdges==e),vesselRadius];
                    adjacencyMatrix(edges{e}(2),edges{e}(1),:) = [e,nnz(labeledEdges==e),vesselRadius];
                end
            end
            
            % Delete nodes without edges
            nodes = [r,c];
            nodes(deletedPoints,:) = [];
            adjacencyMatrix(:,deletedPoints,:) = [];
            adjacencyMatrix(deletedPoints,:,:) = [];
            
            % Filter graph by deleting short leaves
            longestEdge = max(max(adjacencyMatrix(:,:,2)));
            deletedPoints = find((sum(adjacencyMatrix(:,:,2)>0,1)==1).*(sum(adjacencyMatrix(:,:,2),1)<0.1*longestEdge));
            while (~isempty(deletedPoints)),
                nodes(deletedPoints,:) = [];
                adjacencyMatrix(:,deletedPoints,:) = [];
                adjacencyMatrix(deletedPoints,:,:) = [];
                deletedPoints = find((sum(adjacencyMatrix(:,:,2)>0,1)==1).*(sum(adjacencyMatrix(:,:,2),1)<0.1*longestEdge));
            end
            
            % Find main vessels
            [~,root] = min(sqrt(sum((nodes-repmat(center,size(nodes,1),1)).^2,2)));
            [dist, path, pred] = graphshortestpath(sparse(adjacencyMatrix(:,:,2)),root,'Directed',false,'Method','Dijkstra');
            dist(isinf(dist))=0;
            usedPath = [];
            for i=1:4,
                [~,maxPathIdx(i)] = max(dist .* cellfun(@(x) sum(ismember(x,usedPath))<0.25*length(x),path));
                usedPath = [usedPath,path{maxPathIdx(i)}];
            end
            
            % Collect edges of maximum paths from skeleton
            vImg = zeros(size(labeledEdges));
            for i=1:4,
               e = adjacencyMatrix(sub2ind(size(adjacencyMatrix),path{maxPathIdx(i)}(1:end-1),path{maxPathIdx(i)}(2:end)));
               vImg = vImg + ismember(labeledEdges,e);
            end
            vImg = (vImg>0) .* distTrafo;
            vImg = skeleton .* distTrafo + 10*vImg;
            
            [dataY, dataX, weights] = find(vImg);
            
            result = scaledata(vImg, 0, 255);
            result = uint8(round(result));           
            
%             % Show results
%             figure, imagesc(vesselImage), colormap(jet),
%             hold on, gplot(adjacencyMatrix(:,:,1),[nodes(:,2),nodes(:,1)],'-w'), daspect([1,1,1])
%             figure, imagesc(vImg),colorbar;
       
        case 8
            %% Get every kth pixel in skeleton, compute voronoi
            % diagram and assign mean of voronoi cell to point
    end
    
    % Scale weights to range [1,10] and square it  
    weights = scaledata(weights,1,10);
    if strmatch('quadricWeights',varargin,'exact'),
        weights = weights.^2;
    end
    
    % Save resulting image to file
    imwrite(result, [filename, 'vesselPoints.jpg'])
end

